@@ -1,10 +1,10 @@
 /*
     clear_fat - a tool to clear unused space (for testing purposes)
-    Copyright (C) 2000 Free Software Foundation, Inc.
+    Copyright (C) 2000, 2007-2011 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -13,37 +13,76 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "config.h"
+#include <config.h>
 
 #include <string.h>
 #include <stdio.h>
+#include <stdlib.h>
+#include <limits.h>
+#include <getopt.h>
+#include "closeout.h"
+#include "configmake.h"
+#include "error.h"
+#include "long-options.h"
+#include "progname.h"
+#include "xstrtol.h"
+
+#include "../../libparted/fs/fat/fat.h"
+
+#include <locale.h>
+
+/* Take care of NLS matters.  */
+
+#include "gettext.h"
+#if ! ENABLE_NLS
+# undef textdomain
+# define textdomain(Domainname) /* empty */
+# undef bindtextdomain
+# define bindtextdomain(Domainname, Dirname) /* empty */
+#endif
+
+#undef _
+#define _(msgid) gettext (msgid)
 
 #ifndef DISCOVER_ONLY
 
-#include "../../libparted/fs_fat/fat.h"
+/* The official name of this program (e.g., no `g' prefix).  */
+#define PROGRAM_NAME "clearfat"
 
-static char* help_msg =
-"Usage:  clearfat DEVICE MINOR\n"
-"\n"
-"This program is used to enhance the automated testing.  It is not useful for\n"
-"anything much else.\n";
+#define AUTHORS \
+  "<http://git.debian.org/?p=parted/parted.git;a=blob_plain;f=AUTHORS>"
+
+static void
+usage (int status)
+{
+  if (status != EXIT_SUCCESS)
+    fprintf (stderr, _("Try `%s --help' for more information.\n"),
+	     program_name);
+  else
+    {
+      printf (_("\
+Usage: %s [OPTION]\n\
+  or:  %s DEVICE MINOR\n"), PROGRAM_NAME, PROGRAM_NAME);
+      fputs (_("\
+Clear unused space on a FAT partition (a GNU Parted testing tool).\n\
+\n\
+"), stdout);
+      fputs (_("      --help     display this help and exit\n"), stdout);
+      fputs (_("      --version  output version information and exit\n"),
+             stdout);
+      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
+    }
+  exit (status);
+}
 
 #define CLEAR_BUFFER_SIZE		(1024 * 1024)
 #define CLEAR_BUFFER_SECTORS		(CLEAR_BUFFER_SIZE/512)
 
 static char buffer [CLEAR_BUFFER_SIZE];
 
-static int
-_do_help ()
-{
-	printf (help_msg);
-	exit (1);
-}
-
 /* generic clearing code ***************************************************/
 
 static int
@@ -130,6 +169,10 @@ _calc_fat_entry_offset (PedFileSystem* fs, FatCluster cluster)
 	FatSpecific*	fs_info = FAT_SPECIFIC (fs);
 
 	switch (fs_info->fat_type) {
+                case FAT_TYPE_FAT12:
+                        PED_ASSERT (0);
+                        break;
+
 		case FAT_TYPE_FAT16:
 			return cluster * 2;
 
@@ -224,8 +267,36 @@ main (int argc, char* argv[])
 	PedPartition*		part;
 	PedFileSystem*		fs;
 
-	if (argc < 3)
-		_do_help ();
+        set_program_name (argv[0]);
+        setlocale (LC_ALL, "");
+        bindtextdomain (PACKAGE, LOCALEDIR);
+        textdomain (PACKAGE);
+
+	atexit (close_stdout);
+
+	parse_long_options (argc, argv, PROGRAM_NAME, PACKAGE, VERSION,
+                            usage, AUTHORS, (char const *) NULL);
+	if (getopt_long (argc, argv, "", NULL, NULL) != -1)
+	  usage (EXIT_FAILURE);
+
+	if (argc - optind < 2)
+          {
+            error (0, 0, _("too few arguments"));
+            usage (EXIT_FAILURE);
+          }
+	if (2 < argc - optind)
+          {
+            error (0, 0, _("too many arguments"));
+            usage (EXIT_FAILURE);
+          }
+
+	unsigned long minor_dev_number;
+	if (xstrtoul (argv[2], NULL, 10, &minor_dev_number, NULL)
+	    || INT_MAX < minor_dev_number)
+	  {
+	    error (0, 0, _("invalid minor device number: %s"), argv[2]);
+            usage (EXIT_FAILURE);
+	  }
 
 	dev = ped_device_get (argv [1]);
 	if (!dev)
@@ -236,8 +307,8 @@ main (int argc, char* argv[])
 	disk = ped_disk_new (dev);
 	if (!disk)
 		goto error_close_dev;
- 
-	part = ped_disk_get_partition (disk, atoi (argv[2]));
+
+	part = ped_disk_get_partition (disk, minor_dev_number);
 	if (!part) {
 		printf ("Couldn't find partition `%s'\n", argv[2]);
 		goto error_destroy_disk;
@@ -280,4 +351,3 @@ main()
 }
 
 #endif /* DISCOVER_ONLY */
-
