@@ -1,10 +1,10 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 2000 Free Software Foundation, Inc.
+    Copyright (C) 2000, 2007-2011 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -13,8 +13,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 /**
@@ -31,6 +30,7 @@
  * @{
  */
 
+#include <config.h>
 #include <stdlib.h>
 #include <parted/parted.h>
 #include <parted/debug.h>
@@ -56,7 +56,7 @@ const PedAlignment* ped_alignment_none = NULL;
  * is all about adding or subtracting N, and we like our remainders
  * to be between 0 and N - 1.
  */
-PedSector
+static PedSector
 abs_mod (PedSector a, PedSector b)
 {
 	if (a < 0)
@@ -74,18 +74,6 @@ ped_round_down_to (PedSector sector, PedSector grain_size)
 	return sector - abs_mod (sector, grain_size);
 }
 
-inline PedSector
-ped_div_round_up (PedSector numerator, PedSector divisor)
-{
-                return (numerator + divisor - 1) / divisor;
-}
-
-inline PedSector
-ped_div_round_to_nearest (PedSector numerator, PedSector divisor)
-{
-                return (numerator + divisor/2) / divisor;
-}
-
 /* Rounds a number up to the closest number that is a multiple of
  * grain_size.
  */
@@ -114,8 +102,8 @@ ped_round_to_nearest (PedSector sector, PedSector grain_size)
 PedSector
 ped_greatest_common_divisor (PedSector a, PedSector b)
 {
-	PED_ASSERT (a >= 0, return 0);
-	PED_ASSERT (b >= 0, return 0);
+	PED_ASSERT (a >= 0);
+	PED_ASSERT (b >= 0);
 
 	/* Put the arguments in the "right" format.  (Recursive calls made by
 	 * this function are always in the right format.)
@@ -133,13 +121,13 @@ ped_greatest_common_divisor (PedSector a, PedSector b)
  * Initialize a preallocated piece of memory for an alignment object
  * (used by PedConstraint).
  *
- * The object will represent all sectors \e s for which the equation 
+ * The object will represent all sectors \e s for which the equation
  * <tt>s = offset + X * grain_size</tt> holds.
  */
 int
 ped_alignment_init (PedAlignment* align, PedSector offset, PedSector grain_size)
 {
-	PED_ASSERT (align != NULL, return 0);
+	PED_ASSERT (align != NULL);
 
 	if (grain_size < 0)
 		return 0;
@@ -172,7 +160,7 @@ ped_alignment_new (PedSector offset, PedSector grain_size)
 	return align;
 
 error_free_align:
-	ped_free (align);
+	free (align);
 error:
 	return NULL;
 }
@@ -183,8 +171,7 @@ error:
 void
 ped_alignment_destroy (PedAlignment* align)
 {
-	if (align)
-		ped_free (align);
+	free (align);
 }
 
 /**
@@ -209,7 +196,7 @@ ped_alignment_duplicate (const PedAlignment* align)
  * 	gcd = greatest common divisor of a and b
  * 	gcd = x*a + y*b
  */
-EuclidTriple
+static EuclidTriple
 extended_euclid (int a, int b)
 {
 	EuclidTriple	result;
@@ -234,7 +221,7 @@ extended_euclid (int a, int b)
  * intersection of two alignments.  That is, a sector satisfies the
  * new alignment object if and only if it satisfies both of the original
  * ones.  (See ped_alignment_is_aligned() for the meaning of "satisfies")
- * 
+ *
  * Apart from the trivial cases (where one or both of the alignment objects
  * constraints have no sectors that satisfy them), this is what we're trying to
  * do:
@@ -244,18 +231,18 @@ extended_euclid (int a, int b)
  *  - hard part - solve the simultaneous equations, for offset, where offset,
  *  X and Y are variables.  (Note: offset can be obtained from either X or Y,
  *  by substituing into either equation)
- * 
+ *
  * \code
  *  	offset = \p a->offset + X * \p a->grain_size		(1)
  *  	offset = \p b->offset + Y * \p b->grain_size		(2)
  * \endcode
- * 
+ *
  * or, abbreviated:
  *
  * \code
  *  	o = Ao + X*Ag		(1)
  *  	o = Bo + Y*Bg		(2)
- * 
+ *
  *  =>	Ao + X*Ag    = Bo + Y*Bg     (1) = (2)
  *  	X*Ag - Y*Bg  = Bo - Ao  (3)
  * \endcode
@@ -273,9 +260,9 @@ extended_euclid (int a, int b)
  * \endcode
  *
  * gcd is a factor of the linear combination.  QED
- * 
- * Anyway, \p a * Ag + \p b * Bg = gcd can be solved (for \p a, \p b and gcd) 
- * with Euclid's extended algorithm.  Then, we just multiply through by 
+ *
+ * Anyway, \p a * Ag + \p b * Bg = gcd can be solved (for \p a, \p b and gcd)
+ * with Euclid's extended algorithm.  Then, we just multiply through by
  * (Bo - Ao) / gcd to get (3).
  *
  * i.e.
@@ -287,7 +274,7 @@ extended_euclid (int a, int b)
  * 	X = A*(Bo-Ao)/gcd
  * 	Y = - B*(Bo-Ao)/gcd
  * \endcode
- * 
+ *
  * then:
  * \code
  *  	o = Ao + X*Ag			(1)
@@ -312,14 +299,14 @@ ped_alignment_intersect (const PedAlignment* a, const PedAlignment* b)
 	PedSector	delta_on_gcd;
 	EuclidTriple	gcd_factors;
 
-        
+
 	if (!a || !b)
 		return NULL;
 
         /*PED_DEBUG (0x10, "intersecting alignments (%d,%d) and (%d,%d)",
                         a->offset, a->grain_size, b->offset, b->grain_size);
         */
-        
+
 	if (a->grain_size < b->grain_size) {
 		const PedAlignment*	tmp;
 	        tmp = a; a = b; b = tmp;
@@ -357,7 +344,7 @@ static PedSector
 _closest_inside_geometry (const PedAlignment* align, const PedGeometry* geom,
 			  PedSector sector)
 {
-	PED_ASSERT (align != NULL, return -1);
+	PED_ASSERT (align != NULL);
 
 	if (!align->grain_size) {
 		if (ped_alignment_is_aligned (align, geom, sector)
@@ -383,7 +370,7 @@ _closest_inside_geometry (const PedAlignment* align, const PedGeometry* geom,
 /**
  * This function returns the closest sector to \p sector that lies inside
  * \p geom that satisfies the given alignment constraint \p align.  It prefers
- * sectors that are beyond \p sector (are not smaller than \p sector), 
+ * sectors that are beyond \p sector (are not smaller than \p sector),
  * but does not guarantee that this.
  *
  * \return a PedSector on success, \c -1 on failure
@@ -394,7 +381,7 @@ ped_alignment_align_up (const PedAlignment* align, const PedGeometry* geom,
 {
 	PedSector	result;
 
-	PED_ASSERT (align != NULL, return -1);
+	PED_ASSERT (align != NULL);
 
 	if (!align->grain_size)
 		result = align->offset;
@@ -422,7 +409,7 @@ ped_alignment_align_down (const PedAlignment* align, const PedGeometry* geom,
 {
 	PedSector	result;
 
-	PED_ASSERT (align != NULL, return -1);
+	PED_ASSERT (align != NULL);
 
 	if (!align->grain_size)
 		result = align->offset;
@@ -452,7 +439,7 @@ closest (PedSector sector, PedSector a, PedSector b)
 }
 
 /**
- * This function returns the sector that is closest to \p sector, 
+ * This function returns the sector that is closest to \p sector,
  * satisfies the \p align constraint and lies inside \p geom.
  *
  * \return a PedSector on success, \c -1 on failure
@@ -461,14 +448,14 @@ PedSector
 ped_alignment_align_nearest (const PedAlignment* align, const PedGeometry* geom,
 			     PedSector sector)
 {
-	PED_ASSERT (align != NULL, return -1);
+	PED_ASSERT (align != NULL);
 
 	return closest (sector, ped_alignment_align_up (align, geom, sector),
 			ped_alignment_align_down (align, geom, sector));
 }
 
 /**
- * This function returns 1 if \p sector satisfies the alignment 
+ * This function returns 1 if \p sector satisfies the alignment
  * constraint \p align and lies inside \p geom.
  *
  * \return \c 1 on success, \c 0 on failure
@@ -492,4 +479,3 @@ ped_alignment_is_aligned (const PedAlignment* align, const PedGeometry* geom,
 /**
  * @}
  */
-
