@@ -1,6 +1,6 @@
 /*
     libparted
-    Copyright (C) 1998, 1999, 2000, 2007 Free Software Foundation, Inc.
+    Copyright (C) 1998-2000, 2007, 2009-2011 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -58,7 +58,7 @@ calc_deltas (FatOpContext* ctx)
 			(int) new_fs_info->cluster_sectors);
 		return 0;
 	}
-	
+
 	ctx->start_move_delta = sector_delta / ctx->frag_sectors;
 
 #ifdef PED_VERBOSE
@@ -108,9 +108,9 @@ fat_op_context_new (PedFileSystem* new_fs, PedFileSystem* old_fs)
 	return ctx;
 
 error_free_buffer_map:
-	ped_free (ctx->buffer_map);
+	free (ctx->buffer_map);
 error_free_ctx:
-	ped_free (ctx);
+	free (ctx);
 error:
 	return NULL;
 }
@@ -118,9 +118,9 @@ error:
 void
 fat_op_context_destroy (FatOpContext* ctx)
 {
-	ped_free (ctx->buffer_map);
-	ped_free (ctx->remap);
-	ped_free (ctx);
+	free (ctx->buffer_map);
+	free (ctx->remap);
+	free (ctx);
 }
 
 FatFragment
@@ -152,7 +152,7 @@ fat_op_context_map_static_cluster (const FatOpContext* ctx, FatCluster clst)
 	FatFragment	mapped_frag;
 
 	mapped_frag = fat_op_context_map_static_fragment (ctx,
-				fat_cluster_to_frag (ctx->old_fs, clst)); 
+				fat_cluster_to_frag (ctx->old_fs, clst));
 	if (mapped_frag != -1)
 		return fat_frag_to_cluster (ctx->new_fs, mapped_frag);
 	else
@@ -171,7 +171,7 @@ fat_op_context_map_cluster (const FatOpContext* ctx, FatCluster clst)
 	FatFragment	mapped_frag;
 
 	mapped_frag = fat_op_context_map_fragment (ctx,
-				fat_cluster_to_frag (ctx->old_fs, clst)); 
+				fat_cluster_to_frag (ctx->old_fs, clst));
 	if (mapped_frag != -1)
 		return fat_frag_to_cluster (ctx->new_fs, mapped_frag);
 	else
@@ -228,7 +228,7 @@ fat_op_context_create_initial_fat (FatOpContext* ctx)
 			continue;
 
 		new_clst = fat_frag_to_cluster (ctx->new_fs, new_frag);
-		PED_ASSERT (new_clst != 0, return 0);
+		PED_ASSERT (new_clst != 0);
 
 		if (frag_flag == FAT_FLAG_BAD) {
 			if (!fat_table_set_bad (new_fs_info->fat, new_clst))
@@ -248,7 +248,7 @@ fat_op_context_create_initial_fat (FatOpContext* ctx)
 			continue;
 
 		clst = fat_sector_to_cluster (ctx->new_fs, new_sect);
-		PED_ASSERT (clst != 0, return 0);
+		PED_ASSERT (clst != 0);
 
 		if (!fat_table_set_eof (new_fs_info->fat, clst))
 			return 0;
