@@ -1,6 +1,6 @@
 /*
     libparted
-    Copyright (C) 1998-2001, 2007-2010 Free Software Foundation, Inc.
+    Copyright (C) 1998-2001, 2007-2011 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -97,8 +97,7 @@ fat_set_frag_sectors (PedFileSystem* fs, PedSector frag_sectors)
 	FatSpecific*	fs_info = FAT_SPECIFIC (fs);
 
 	PED_ASSERT (fs_info->cluster_sectors % frag_sectors == 0
-			&& frag_sectors <= fs_info->cluster_sectors,
-		    return 0);
+			&& frag_sectors <= fs_info->cluster_sectors);
 
 	fs_info->frag_size = frag_sectors * 512;
 	fs_info->frag_sectors = frag_sectors;
