@@ -1,6 +1,6 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 2004, 2005, 2007 Free Software Foundation, Inc.
+    Copyright (C) 2004-2005, 2007, 2009-2011 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -160,7 +160,7 @@ hfsplus_file_find_extent (HfsPPrivateFile* file, PedSector sector,
 	}
 
 	/* ret == {0,0} */
-	PED_ASSERT(file->start_cache && block >= file->start_cache, return ret);
+	PED_ASSERT(file->start_cache && block >= file->start_cache);
 
 	for (s = file->start_cache, i = 0; i < HFSP_EXT_NB; i++) {
 		if ((block >= s) && (block < s + PED_BE32_TO_CPU (
