@@ -1,10 +1,10 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 2004, 2005, 2007 Free Software Foundation, Inc.
+    Copyright (C) 2004-2005, 2007, 2009-2011 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -13,8 +13,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifndef DISCOVER_ONLY
@@ -57,10 +56,10 @@ hfsplus_effect_move_extent (PedFileSystem *fs, unsigned int *ptr_fblock,
 	unsigned int		next_to_fblock;
 	unsigned int		start, stop;
 
-	PED_ASSERT (hfsp_block != NULL, return -1);
-	PED_ASSERT (*ptr_to_fblock <= *ptr_fblock, return -1);
+	PED_ASSERT (hfsp_block != NULL);
+	PED_ASSERT (*ptr_to_fblock <= *ptr_fblock);
 	/* quiet GCC */
-	next_to_fblock = start = stop = 0;
+	start = stop = 0;
 
 /*
 	Try to fit the extent AT or _BEFORE_ the wanted place,
@@ -81,7 +80,7 @@ hfsplus_effect_move_extent (PedFileSystem *fs, unsigned int *ptr_fblock,
 		}
 		ok = (stop-start == size);
 	}
-	
+
 	/* Forward search */
 	/* 1 pass relocation in the gap merged with 2 pass reloc after source */
 	if (!ok && *ptr_to_fblock != *ptr_fblock) {
@@ -196,13 +195,13 @@ hfsplus_do_move (PedFileSystem* fs, unsigned int *ptr_src,
 						ref->ext_length);
 
 	if (new_start == -1) return -1;
-	
+
 	if (ref->ext_start != (unsigned) new_start) {
 		switch (ref->where) {
 		/************ VH ************/
 		    case CR_PRIM_CAT :
 			priv_data->catalog_file
-			->first[ref->ref_index].start_block = 
+			->first[ref->ref_index].start_block =
 				PED_CPU_TO_BE32(new_start);
 			goto CR_PRIM;
 		    case CR_PRIM_EXT :
@@ -283,8 +282,8 @@ hfsplus_do_move (PedFileSystem* fs, unsigned int *ptr_src,
 
 		    CR_BTREE :
 			PED_ASSERT(PED_SECTOR_SIZE_DEFAULT * ref->sect_by_block
-				   > ref->ref_offset, return -1 );
-			if (!hfsplus_file_read(file, hfsp_block, 
+				   > ref->ref_offset);
+			if (!hfsplus_file_read(file, hfsp_block,
 				(PedSector)ref->ref_block * ref->sect_by_block,
 				ref->sect_by_block))
 				return -1;
@@ -313,7 +312,7 @@ hfsplus_do_move (PedFileSystem* fs, unsigned int *ptr_src,
 
 		move = hfsc_cache_move_extent(cache, ref->ext_start, new_start);
 		if (!move) return -1;
-		PED_ASSERT(move == ref, return -1);
+		PED_ASSERT(move == ref);
 	}
 
 	return new_start;
@@ -328,11 +327,11 @@ hfsplus_save_allocation(PedFileSystem *fs)
 	unsigned int		map_sectors, i, j;
 	int			ret = 1;
 
-	map_sectors = ( PED_BE32_TO_CPU (priv_data->vh->total_blocks) 
+	map_sectors = ( PED_BE32_TO_CPU (priv_data->vh->total_blocks)
 			+ PED_SECTOR_SIZE_DEFAULT * 8 - 1 ) / (PED_SECTOR_SIZE_DEFAULT * 8);
 
 	for (i = 0; i < map_sectors;) {
-		for (j = i; 
+		for (j = i;
 		     (TST_BLOC_OCCUPATION(priv_data->dirty_alloc_map,j));
 		     ++j)
 			CLR_BLOC_OCCUPATION(priv_data->dirty_alloc_map,j);
@@ -479,7 +478,6 @@ hfsplus_cache_from_catalog(HfsCPrivateCache* cache, PedFileSystem* fs,
 	uint8_t			node_1[PED_SECTOR_SIZE_DEFAULT];
 	uint8_t*		node;
 	HfsPHeaderRecord*	header;
-	HfsPNodeDescriptor*	desc = (HfsPNodeDescriptor*) node_1;
 	HfsPCatalogKey*		catalog_key;
 	HfsPCatalog*		catalog_data;
 	HfsPExtDescriptor*	extent;
@@ -504,16 +502,16 @@ hfsplus_cache_from_catalog(HfsCPrivateCache* cache, PedFileSystem* fs,
 	leaf_node = PED_BE32_TO_CPU (header->first_leaf_node);
 	bsize = PED_BE16_TO_CPU (header->node_size);
 	size = bsize / PED_SECTOR_SIZE_DEFAULT;
-	PED_ASSERT(size < 256, return 0);
+	PED_ASSERT(size < 256);
 
 	node = (uint8_t*) ped_malloc(bsize);
 	if (!node) return 0;
-	desc = (HfsPNodeDescriptor*) node;
+	HfsPNodeDescriptor *desc = (HfsPNodeDescriptor*) node;
 
 	for (; leaf_node; leaf_node = PED_BE32_TO_CPU (desc->next)) {
 		if (!hfsplus_file_read (priv_data->catalog_file, node,
 					(PedSector) leaf_node * size, size)) {
-			ped_free (node);
+			free (node);
 			return 0;
 		}
 		record_number = PED_BE16_TO_CPU (desc->rec_nb);
@@ -537,7 +535,7 @@ hfsplus_cache_from_catalog(HfsCPrivateCache* cache, PedFileSystem* fs,
 					PED_EXCEPTION_ERROR,
 					PED_EXCEPTION_CANCEL,
 					_("The file system contains errors."));
-				ped_free (node);
+				free (node);
 				return 0;
 			}
 
@@ -568,7 +566,7 @@ hfsplus_cache_from_catalog(HfsCPrivateCache* cache, PedFileSystem* fs,
 					where,
 					j )
 				   ) {
-					ped_free (node);
+					free (node);
 				 	return 0;
 				}
 			}
@@ -586,14 +584,14 @@ hfsplus_cache_from_catalog(HfsCPrivateCache* cache, PedFileSystem* fs,
 					CR_BTREE_CAT,
 					j )
 				   ) {
-					ped_free (node);
+					free (node);
 				 	return 0;
 				}
 			}
 		}
 	}
 
-	ped_free (node);
+	free (node);
 	return 1;
 }
 
@@ -606,7 +604,6 @@ hfsplus_cache_from_extent(HfsCPrivateCache* cache, PedFileSystem* fs,
 	uint8_t			node_1[PED_SECTOR_SIZE_DEFAULT];
 	uint8_t*		node;
 	HfsPHeaderRecord*	header;
-	HfsPNodeDescriptor*	desc = (HfsPNodeDescriptor*) node_1;
 	HfsPExtentKey*		extent_key;
 	HfsPExtDescriptor*	extent;
 	unsigned int		leaf_node, record_number;
@@ -627,16 +624,16 @@ hfsplus_cache_from_extent(HfsCPrivateCache* cache, PedFileSystem* fs,
 	leaf_node = PED_BE32_TO_CPU (header->first_leaf_node);
 	bsize = PED_BE16_TO_CPU (header->node_size);
 	size = bsize / PED_SECTOR_SIZE_DEFAULT;
-	PED_ASSERT(size < 256, return 0);
+	PED_ASSERT(size < 256);
 
 	node = (uint8_t*) ped_malloc (bsize);
 	if (!node) return -1;
-	desc = (HfsPNodeDescriptor*) node;
+	HfsPNodeDescriptor *desc = (HfsPNodeDescriptor*) node;
 
 	for (; leaf_node; leaf_node = PED_BE32_TO_CPU (desc->next)) {
 		if (!hfsplus_file_read (priv_data->extents_file, node,
 					(PedSector) leaf_node * size, size)) {
-			ped_free (node);
+			free (node);
 			return 0;
 		}
 		record_number = PED_BE16_TO_CPU (desc->rec_nb);
@@ -656,10 +653,10 @@ hfsplus_cache_from_extent(HfsCPrivateCache* cache, PedFileSystem* fs,
 					PED_EXCEPTION_ERROR,
 					PED_EXCEPTION_CANCEL,
 					_("The file system contains errors."));
-				ped_free (node);
+				free (node);
 				return -1;
 			}
-			
+
 			switch (extent_key->file_ID) {
 			    case PED_CPU_TO_BE32 (HFS_XTENT_ID) :
 				if (ped_exception_throw (
@@ -688,7 +685,7 @@ hfsplus_cache_from_extent(HfsCPrivateCache* cache, PedFileSystem* fs,
 			    	where = CR_BTREE_EXT_0;
 				break;
 			}
-			
+
 			for (j = 0; j < HFSP_EXT_NB; ++j) {
 				if (!extent[j].block_count) break;
 				if (!hfsc_cache_add_extent(
@@ -701,14 +698,14 @@ hfsplus_cache_from_extent(HfsCPrivateCache* cache, PedFileSystem* fs,
 					where,
 					j )
 				   ) {
-					ped_free (node);
+					free (node);
 					return 0;
 				}
 			}
 		}
 	}
-	
-	ped_free (node);
+
+	free (node);
 	return 1;
 }
 
@@ -721,7 +718,6 @@ hfsplus_cache_from_attributes(HfsCPrivateCache* cache, PedFileSystem* fs,
 	uint8_t			node_1[PED_SECTOR_SIZE_DEFAULT];
 	uint8_t*		node;
 	HfsPHeaderRecord*	header;
-	HfsPNodeDescriptor*	desc = (HfsPNodeDescriptor*) node_1;
 	HfsPPrivateGenericKey*	generic_key;
 	HfsPForkDataAttr*	fork_ext_data;
 	HfsPExtDescriptor*	extent;
@@ -739,16 +735,16 @@ hfsplus_cache_from_attributes(HfsCPrivateCache* cache, PedFileSystem* fs,
 	leaf_node = PED_BE32_TO_CPU (header->first_leaf_node);
 	bsize = PED_BE16_TO_CPU (header->node_size);
 	size = bsize / PED_SECTOR_SIZE_DEFAULT;
-	PED_ASSERT(size < 256, return 0);
+	PED_ASSERT(size < 256);
 
 	node = (uint8_t*) ped_malloc(bsize);
 	if (!node) return 0;
-	desc = (HfsPNodeDescriptor*) node;
+	HfsPNodeDescriptor *desc = (HfsPNodeDescriptor*) node;
 
 	for (; leaf_node; leaf_node = PED_BE32_TO_CPU (desc->next)) {
 		if (!hfsplus_file_read (priv_data->attributes_file, node,
 					(PedSector) leaf_node * size, size)) {
-			ped_free (node);
+			free (node);
 			return 0;
 		}
 		record_number = PED_BE16_TO_CPU (desc->rec_nb);
@@ -770,7 +766,7 @@ hfsplus_cache_from_attributes(HfsCPrivateCache* cache, PedFileSystem* fs,
 					PED_EXCEPTION_ERROR,
 					PED_EXCEPTION_CANCEL,
 					_("The file system contains errors."));
-				ped_free (node);
+				free (node);
 				return 0;
 			}
 
@@ -791,7 +787,7 @@ hfsplus_cache_from_attributes(HfsCPrivateCache* cache, PedFileSystem* fs,
 						CR_BTREE_ATTR,
 						j )
 					   ) {
-						ped_free(node);
+						free(node);
 						return 0;
 					}
 				}
@@ -812,7 +808,7 @@ hfsplus_cache_from_attributes(HfsCPrivateCache* cache, PedFileSystem* fs,
 						CR_BTREE_ATTR,
 						j )
 					   ) {
-						ped_free(node);
+						free(node);
 						return 0;
 					}
 				}
@@ -820,7 +816,7 @@ hfsplus_cache_from_attributes(HfsCPrivateCache* cache, PedFileSystem* fs,
 		}
 	}
 
-	ped_free (node);
+	free (node);
 	return 1;
 }
 
@@ -870,7 +866,7 @@ hfsplus_pack_free_space_from_block (PedFileSystem *fs, unsigned int fblock,
 				          + 1 - start - to_free;
 	int			ret;
 
-	PED_ASSERT (!hfsp_block, return 0);
+	PED_ASSERT (!hfsp_block);
 
 	cache = hfsplus_cache_extents (fs, timer);
 	if (!cache)
@@ -932,12 +928,12 @@ hfsplus_pack_free_space_from_block (PedFileSystem *fs, unsigned int fblock,
 		ped_timer_update(timer, (float)(to_fblock - start) / divisor);
 	}
 
-	ped_free (hfsp_block); hfsp_block = NULL; hfsp_block_count = 0;
+	free (hfsp_block); hfsp_block = NULL; hfsp_block_count = 0;
 	hfsc_delete_cache (cache);
 	return 1;
 
 error_alloc:
-	ped_free (hfsp_block); hfsp_block = NULL; hfsp_block_count = 0;
+	free (hfsp_block); hfsp_block = NULL; hfsp_block_count = 0;
 error_cache:
 	hfsc_delete_cache (cache);
 	return 0;
