@@ -1,6 +1,7 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1999-2001, 2004-2005, 2007-2009 Free Software Foundation, Inc.
+    Copyright (C) 1999-2001, 2004-2005, 2007-2011 Free Software Foundation,
+    Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -80,11 +81,13 @@ static const char MBR_BOOT_CODE[] = {
 #define PARTITION_FAT16_LBA_H	(PARTITION_FAT16_LBA	| PART_FLAG_HIDDEN)
 
 #define PARTITION_COMPAQ_DIAG	0x12
+#define PARTITION_MSFT_RECOVERY	0x27
 #define PARTITION_LDM		0x42
 #define PARTITION_LINUX_SWAP	0x82
 #define PARTITION_LINUX		0x83
 #define PARTITION_LINUX_EXT	0x85
 #define PARTITION_LINUX_LVM	0x8e
+#define PARTITION_HFS		0xaf
 #define PARTITION_SUN_UFS	0xbf
 #define PARTITION_DELL_DIAG	0xde
 #define PARTITION_GPT		0xee
@@ -155,11 +158,79 @@ typedef struct {
 	int		lba;
 	int		palo;
 	int		prep;
+	int		diag;
 	OrigState*	orig;			/* used for CHS stuff */
 } DosPartitionData;
 
 static PedDiskType msdos_disk_type;
 
+#if 0
+From http://www.win.tue.nl/~aeb/linux/fs/fat/fat-1.html
+
+The 2-byte numbers are stored little endian (low order byte first).
+
+Here the FAT12 version, that is also the common part of the FAT12, FAT16 and FAT32 boot sectors. See further below.
+
+Bytes   Content
+0-2     Jump to bootstrap (E.g. eb 3c 90; on i86: JMP 003E NOP.
+        One finds either eb xx 90, or e9 xx xx.
+        The position of the bootstrap varies.)
+3-10    OEM name/version (E.g. "IBM  3.3", "IBM 20.0", "MSDOS5.0", "MSWIN4.0".
+        Various format utilities leave their own name, like "CH-FOR18".
+        Sometimes just garbage. Microsoft recommends "MSWIN4.1".)
+        /* BIOS Parameter Block starts here */
+11-12   Number of bytes per sector (512)
+        Must be one of 512, 1024, 2048, 4096.
+13      Number of sectors per cluster (1)
+        Must be one of 1, 2, 4, 8, 16, 32, 64, 128.
+        A cluster should have at most 32768 bytes. In rare cases 65536 is OK.
+14-15   Number of reserved sectors (1)
+        FAT12 and FAT16 use 1. FAT32 uses 32.
+16      Number of FAT copies (2)
+17-18   Number of root directory entries (224)
+        0 for FAT32. 512 is recommended for FAT16.
+19-20   Total number of sectors in the filesystem (2880)
+        (in case the partition is not FAT32 and smaller than 32 MB)
+21      Media descriptor type (f0: 1.4 MB floppy, f8: hard disk; see below)
+22-23   Number of sectors per FAT (9)
+        0 for FAT32.
+24-25   Number of sectors per track (12)
+26-27   Number of heads (2, for a double-sided diskette)
+28-29   Number of hidden sectors (0)
+        Hidden sectors are sectors preceding the partition.
+        /* BIOS Parameter Block ends here */
+30-509  Bootstrap
+510-511 Signature 55 aa
+#endif
+
+/* There is a significant risk of misclassifying (as msdos)
+   a disk that is composed solely of a single FAT partition.
+   Return false if sector S could not be a valid FAT boot sector.
+   Otherwise, return true.  */
+static bool
+maybe_FAT (unsigned char const *s)
+{
+  if (! (s[0] == 0xeb || s[0] == 0xe9))
+    return false;
+
+  unsigned int sector_size = PED_LE16_TO_CPU (*(uint16_t *) (s + 11));
+  switch (sector_size)
+    {
+    case 512:
+    case 1024:
+    case 2048:
+    case 4096:
+      break;
+    default:
+      return false;
+    }
+
+  if (! (s[21] == 0xf0 || s[21] == 0xf8))
+    return false;
+
+  return true;
+}
+
 static int
 msdos_probe (const PedDevice *dev)
 {
@@ -167,7 +238,7 @@ msdos_probe (const PedDevice *dev)
 	DosRawTable*	part_table;
 	int		i;
 
-	PED_ASSERT (dev != NULL, return 0);
+	PED_ASSERT (dev != NULL);
 
         if (dev->sector_size < sizeof *part_table)
                 return 0;
@@ -187,12 +258,20 @@ msdos_probe (const PedDevice *dev)
 	 * and ensure that each partition has a boot indicator that is
 	 * either 0 or 0x80.
 	 */
+	unsigned int n_active = 0;
 	for (i = 0; i < DOS_N_PRI_PARTITIONS; i++) {
+		if (part_table->partitions[i].boot_ind == 0x80)
+			++n_active;
 		if (part_table->partitions[i].boot_ind != 0
 		    && part_table->partitions[i].boot_ind != 0x80)
 			goto probe_fail;
 	}
 
+	/* If there are no active partitions and this is probably
+	   a FAT file system, do not classify it as msdos.  */
+	if (n_active == 0 && maybe_FAT (label))
+	  goto probe_fail;
+
 	/* If this is a GPT disk, fail here */
 	for (i = 0; i < DOS_N_PRI_PARTITIONS; i++) {
 		if (part_table->partitions[i].type == PARTITION_GPT)
@@ -228,7 +307,7 @@ static PedDisk*
 msdos_alloc (const PedDevice* dev)
 {
 	PedDisk* disk;
-	PED_ASSERT (dev != NULL, return NULL);
+	PED_ASSERT (dev != NULL);
 
 	disk = _ped_disk_alloc ((PedDevice*)dev, &msdos_disk_type);
         if (disk) {
@@ -262,7 +341,7 @@ msdos_duplicate (const PedDisk* disk)
 static void
 msdos_free (PedDisk* disk)
 {
-	PED_ASSERT (disk != NULL, return);
+	PED_ASSERT (disk != NULL);
 
 	DosDiskData *disk_specific = disk->disk_specific;
 	_ped_disk_free (disk);
@@ -305,25 +384,6 @@ msdos_disk_is_flag_available (const PedDisk *disk, PedDiskFlag flag)
         }
 }
 
-#ifndef DISCOVER_ONLY
-static int
-msdos_clobber (PedDevice* dev)
-{
-	PED_ASSERT (dev != NULL, return 0);
-	PED_ASSERT (msdos_probe (dev), return 0);
-
-	void *label;
-	if (!ptt_read_sector (dev, 0, &label))
-		return 0;
-
-	DosRawTable *table = label;
-	table->magic = 0;
-        int write_ok = ped_device_write (dev, (void*) table, 0, 1);
-        free (label);
-	return write_ok;
-}
-#endif /* !DISCOVER_ONLY */
-
 static int
 chs_get_cylinder (const RawCHS* chs)
 {
@@ -351,8 +411,8 @@ chs_to_sector (const PedDevice* dev, const PedCHSGeometry *bios_geom,
 	PedSector	h;		/* lots of bits */
 	PedSector	s;
 
-	PED_ASSERT (bios_geom != NULL, return 0);
-	PED_ASSERT (chs != NULL, return 0);
+	PED_ASSERT (bios_geom != NULL);
+	PED_ASSERT (chs != NULL);
 
 	c = chs_get_cylinder (chs);
 	h = chs_get_head (chs);
@@ -371,8 +431,8 @@ sector_to_chs (const PedDevice* dev, const PedCHSGeometry* bios_geom,
 {
 	PedSector	real_c, real_h, real_s;
 
-	PED_ASSERT (dev != NULL, return);
-	PED_ASSERT (chs != NULL, return);
+	PED_ASSERT (dev != NULL);
+	PED_ASSERT (chs != NULL);
 
 	if (!bios_geom)
 		bios_geom = &dev->bios_geom;
@@ -396,8 +456,8 @@ static PedSector
 legacy_start (const PedDisk* disk, const PedCHSGeometry* bios_geom,
 	      const DosRawPartition* raw_part)
 {
-	PED_ASSERT (disk != NULL, return 0);
-	PED_ASSERT (raw_part != NULL, return 0);
+	PED_ASSERT (disk != NULL);
+	PED_ASSERT (raw_part != NULL);
 
 	return chs_to_sector (disk->dev, bios_geom, &raw_part->chs_start);
 }
@@ -406,8 +466,8 @@ static PedSector
 legacy_end (const PedDisk* disk, const PedCHSGeometry* bios_geom,
 	    const DosRawPartition* raw_part)
 {
-	PED_ASSERT (disk != NULL, return 0);
-	PED_ASSERT (raw_part != NULL, return 0);
+	PED_ASSERT (disk != NULL);
+	PED_ASSERT (raw_part != NULL);
 
 	return chs_to_sector (disk->dev, bios_geom, &raw_part->chs_end);
 }
@@ -416,8 +476,8 @@ static PedSector
 linear_start (const PedDisk* disk, const DosRawPartition* raw_part,
 	      PedSector offset)
 {
-	PED_ASSERT (disk != NULL, return 0);
-	PED_ASSERT (raw_part != NULL, return 0);
+	PED_ASSERT (disk != NULL);
+	PED_ASSERT (raw_part != NULL);
 
 	return offset + PED_LE32_TO_CPU (raw_part->start);
 }
@@ -426,8 +486,8 @@ static PedSector
 linear_end (const PedDisk* disk, const DosRawPartition* raw_part,
 	    PedSector offset)
 {
-	PED_ASSERT (disk != NULL, return 0);
-	PED_ASSERT (raw_part != NULL, return 0);
+	PED_ASSERT (disk != NULL);
+	PED_ASSERT (raw_part != NULL);
 
 	return (linear_start (disk, raw_part, offset)
                 + (PED_LE32_TO_CPU (raw_part->length) - 1));
@@ -441,9 +501,9 @@ partition_check_bios_geometry (PedPartition* part, PedCHSGeometry* bios_geom)
 	DosPartitionData*	dos_data;
 	PedDisk*		disk;
 
-	PED_ASSERT (part != NULL, return 0);
-	PED_ASSERT (part->disk != NULL, return 0);
-	PED_ASSERT (part->disk_specific != NULL, return 0);
+	PED_ASSERT (part != NULL);
+	PED_ASSERT (part->disk != NULL);
+	PED_ASSERT (part->disk_specific != NULL);
 	dos_data = part->disk_specific;
 
 	if (!dos_data->orig)
@@ -465,7 +525,7 @@ disk_check_bios_geometry (const PedDisk* disk, PedCHSGeometry* bios_geom)
 {
 	PedPartition* part = NULL;
 
-	PED_ASSERT (disk != NULL, return 0);
+	PED_ASSERT (disk != NULL);
 
 	while ((part = ped_disk_next_partition (disk, part))) {
 		if (ped_partition_is_active (part)) {
@@ -488,12 +548,11 @@ probe_filesystem_for_geom (const PedPartition* part, PedCHSGeometry* bios_geom)
 	int heads;
 	int res = 0;
 
-	PED_ASSERT (bios_geom        != NULL, return 0);
-        PED_ASSERT (part             != NULL, return 0);
-        PED_ASSERT (part->disk       != NULL, return 0);
-        PED_ASSERT (part->disk->dev  != NULL, return 0);
-        PED_ASSERT (part->disk->dev->sector_size % PED_SECTOR_SIZE_DEFAULT == 0,
-                    return 0);
+	PED_ASSERT (bios_geom        != NULL);
+        PED_ASSERT (part             != NULL);
+        PED_ASSERT (part->disk       != NULL);
+        PED_ASSERT (part->disk->dev  != NULL);
+        PED_ASSERT (part->disk->dev->sector_size % PED_SECTOR_SIZE_DEFAULT == 0);
 
         buf = ped_malloc (part->disk->dev->sector_size);
 
@@ -579,9 +638,9 @@ probe_partition_for_geom (const PedPartition* part, PedCHSGeometry* bios_geom)
 	PedSector cyl_size, head_size;
 	PedSector cylinders, heads, sectors;
 
-	PED_ASSERT (part != NULL, return 0);
-	PED_ASSERT (part->disk_specific != NULL, return 0);
-	PED_ASSERT (bios_geom != NULL, return 0);
+	PED_ASSERT (part != NULL);
+	PED_ASSERT (part->disk_specific != NULL);
+	PED_ASSERT (bios_geom != NULL);
 
 	dos_data = part->disk_specific;
 
@@ -660,8 +719,10 @@ probe_partition_for_geom (const PedPartition* part, PedCHSGeometry* bios_geom)
 	if (cyl_size * denum != a_*H - A_*h)
 		return 0;
 
-	PED_ASSERT (cyl_size > 0, return 0);
- 	PED_ASSERT (cyl_size <= 255 * 63, return 0);
+	if (!(cyl_size > 0))
+		return 0;
+	if (!(cyl_size <= 255 * 63))
+		return 0;
 
 	if (h > 0)
 		head_size = ( a_ - c * cyl_size ) / h;
@@ -669,21 +730,27 @@ probe_partition_for_geom (const PedPartition* part, PedCHSGeometry* bios_geom)
 		head_size = ( A_ - C * cyl_size ) / H;
 	else {
 		/* should not happen because denum != 0 */
-		PED_ASSERT (0, return 0);
+		PED_ASSERT (0);
 	}
 
-	PED_ASSERT (head_size > 0, return 0);
-	PED_ASSERT (head_size <= 63, return 0);
+	if (!(head_size > 0))
+		return 0;
+	if (!(head_size <= 63))
+		return 0;
 
 	cylinders = part->disk->dev->length / cyl_size;
 	heads = cyl_size / head_size;
 	sectors = head_size;
 
-	PED_ASSERT (heads > 0, return 0);
-	PED_ASSERT (heads < 256, return 0);
+	if (!(heads > 0))
+		return 0;
+	if (!(heads < 256))
+		return 0;
 
-	PED_ASSERT (sectors > 0, return 0);
-	PED_ASSERT (sectors <= 63, return 0);
+	if (!(sectors > 0))
+		return 0;
+	if (!(sectors <= 63))
+		return 0;
 
 	/* Some broken OEM partitioning program(s) seem to have an out-by-one
 	 * error on the end of partitions.  We should offer to fix the
@@ -692,8 +759,10 @@ probe_partition_for_geom (const PedPartition* part, PedCHSGeometry* bios_geom)
 	if (((C + 1) * heads + H) * sectors + S == A)
 		C++;
 
-	PED_ASSERT ((c * heads + h) * sectors + s == a, return 0);
-	PED_ASSERT ((C * heads + H) * sectors + S == A, return 0);
+	if (!((c * heads + h) * sectors + s == a))
+		return 0;
+	if (!((C * heads + H) * sectors + S == A))
+		return 0;
 
 	bios_geom->cylinders = cylinders;
 	bios_geom->heads = heads;
@@ -706,9 +775,9 @@ static void
 partition_probe_bios_geometry (const PedPartition* part,
                                PedCHSGeometry* bios_geom)
 {
-	PED_ASSERT (part != NULL, return);
-	PED_ASSERT (part->disk != NULL, return);
-	PED_ASSERT (bios_geom != NULL, return);
+	PED_ASSERT (part != NULL);
+	PED_ASSERT (part->disk != NULL);
+	PED_ASSERT (bios_geom != NULL);
 
 	if (ped_partition_is_active (part)) {
 		if (probe_partition_for_geom (part, bios_geom))
@@ -721,7 +790,7 @@ partition_probe_bios_geometry (const PedPartition* part,
 	if (part->type & PED_PARTITION_LOGICAL) {
 		PedPartition* ext_part;
 		ext_part = ped_disk_extended_partition (part->disk);
-		PED_ASSERT (ext_part != NULL, return);
+		PED_ASSERT (ext_part != NULL);
 		partition_probe_bios_geometry (ext_part, bios_geom);
 	} else {
 		*bios_geom = part->disk->dev->bios_geom;
@@ -769,7 +838,7 @@ disk_probe_bios_geometry (const PedDisk* disk, PedCHSGeometry* bios_geom)
 static int
 raw_part_is_extended (const DosRawPartition* raw_part)
 {
-	PED_ASSERT (raw_part != NULL, return 0);
+	PED_ASSERT (raw_part != NULL);
 
 	switch (raw_part->type) {
 	case PARTITION_DOS_EXT:
@@ -787,7 +856,7 @@ raw_part_is_extended (const DosRawPartition* raw_part)
 static int
 raw_part_is_hidden (const DosRawPartition* raw_part)
 {
-	PED_ASSERT (raw_part != NULL, return 0);
+	PED_ASSERT (raw_part != NULL);
 
 	switch (raw_part->type) {
 	case PARTITION_FAT12_H:
@@ -809,7 +878,7 @@ raw_part_is_hidden (const DosRawPartition* raw_part)
 static int
 raw_part_is_lba (const DosRawPartition* raw_part)
 {
-	PED_ASSERT (raw_part != NULL, return 0);
+	PED_ASSERT (raw_part != NULL);
 
 	switch (raw_part->type) {
 	case PARTITION_FAT32_LBA:
@@ -833,8 +902,8 @@ raw_part_parse (const PedDisk* disk, const DosRawPartition* raw_part,
 	PedPartition* part;
 	DosPartitionData* dos_data;
 
-	PED_ASSERT (disk != NULL, return NULL);
-	PED_ASSERT (raw_part != NULL, return NULL);
+	PED_ASSERT (disk != NULL);
+	PED_ASSERT (raw_part != NULL);
 
 	part = ped_partition_new (
 		disk, type, NULL,
@@ -845,6 +914,9 @@ raw_part_parse (const PedDisk* disk, const DosRawPartition* raw_part,
 	dos_data = part->disk_specific;
 	dos_data->system = raw_part->type;
 	dos_data->boot = raw_part->boot_ind != 0;
+	dos_data->diag = raw_part->type == PARTITION_COMPAQ_DIAG ||
+			 raw_part->type == PARTITION_MSFT_RECOVERY ||
+			 raw_part->type == PARTITION_DELL_DIAG;
 	dos_data->hidden = raw_part_is_hidden (raw_part);
 	dos_data->raid = raw_part->type == PARTITION_LINUX_RAID;
 	dos_data->lvm = raw_part->type == PARTITION_LINUX_LVM_OLD
@@ -872,10 +944,9 @@ read_table (PedDisk* disk, PedSector sector, int is_extended_table)
 	PedPartition*		part;
 	PedPartitionType	type;
 	PedSector		lba_offset;
-	PedConstraint*		constraint_exact;
 
-	PED_ASSERT (disk != NULL, return 0);
-	PED_ASSERT (disk->dev != NULL, return 0);
+	PED_ASSERT (disk != NULL);
+	PED_ASSERT (disk->dev != NULL);
 
 	void *label = NULL;
 	if (!ptt_read_sector (disk->dev, sector, &label))
@@ -943,10 +1014,12 @@ read_table (PedDisk* disk, PedSector sector, int is_extended_table)
 		if (type != PED_PARTITION_EXTENDED)
 			part->fs_type = ped_file_system_probe (&part->geom);
 
-		constraint_exact = ped_constraint_exact (&part->geom);
-		if (!ped_disk_add_partition (disk, part, constraint_exact))
-			goto error;
+		PedConstraint *constraint_exact
+		  = ped_constraint_exact (&part->geom);
+		bool ok = ped_disk_add_partition (disk, part, constraint_exact);
 		ped_constraint_destroy (constraint_exact);
+		if (!ok)
+			goto error;
 
 		/* non-nested extended partition */
 		if (part->type == PED_PARTITION_EXTENDED) {
@@ -991,8 +1064,8 @@ error:
 static int
 msdos_read (PedDisk* disk)
 {
-	PED_ASSERT (disk != NULL, return 0);
-	PED_ASSERT (disk->dev != NULL, return 0);
+	PED_ASSERT (disk != NULL);
+	PED_ASSERT (disk->dev != NULL);
 
 	ped_disk_delete_all (disk);
 	if (!read_table (disk, 0, 0))
@@ -1027,8 +1100,8 @@ fill_raw_part (DosRawPartition* raw_part,
 	DosPartitionData*	dos_data;
 	PedCHSGeometry		bios_geom;
 
-	PED_ASSERT (raw_part != NULL, return 0);
-	PED_ASSERT (part != NULL, return 0);
+	PED_ASSERT (raw_part != NULL);
+	PED_ASSERT (part != NULL);
 
 	partition_probe_bios_geometry (part, &bios_geom);
 
@@ -1060,9 +1133,9 @@ fill_ext_raw_part_geom (DosRawPartition* raw_part,
                         const PedCHSGeometry* bios_geom,
 			const PedGeometry* geom, PedSector offset)
 {
-	PED_ASSERT (raw_part != NULL, return 0);
-	PED_ASSERT (geom != NULL, return 0);
-	PED_ASSERT (geom->dev != NULL, return 0);
+	PED_ASSERT (raw_part != NULL);
+	PED_ASSERT (geom != NULL);
+	PED_ASSERT (geom->dev != NULL);
 
 	raw_part->boot_ind = 0;
 	raw_part->type = PARTITION_DOS_EXT;
@@ -1083,9 +1156,9 @@ write_ext_table (const PedDisk* disk,
 	PedPartition*		part;
 	PedSector		lba_offset;
 
-	PED_ASSERT (disk != NULL, return 0);
-	PED_ASSERT (ped_disk_extended_partition (disk) != NULL, return 0);
-	PED_ASSERT (logical != NULL, return 0);
+	PED_ASSERT (disk != NULL);
+	PED_ASSERT (ped_disk_extended_partition (disk) != NULL);
+	PED_ASSERT (logical != NULL);
 
 	lba_offset = ped_disk_extended_partition (disk)->geom.start;
 
@@ -1131,7 +1204,7 @@ write_empty_table (const PedDisk* disk, PedSector sector)
 	DosRawTable		table;
 	void*			table_sector;
 
-	PED_ASSERT (disk != NULL, return 0);
+	PED_ASSERT (disk != NULL);
 
 	if (ptt_read_sector (disk->dev, sector, &table_sector)) {
 		memcpy (&table, table_sector, sizeof (table));
@@ -1152,7 +1225,7 @@ write_extended_partitions (const PedDisk* disk)
 	PedPartition*		part;
 	PedCHSGeometry		bios_geom;
 
-	PED_ASSERT (disk != NULL, return 0);
+	PED_ASSERT (disk != NULL);
 
 	ext_part = ped_disk_extended_partition (disk);
 	partition_probe_bios_geometry (ext_part, &bios_geom);
@@ -1179,8 +1252,8 @@ msdos_write (const PedDisk* disk)
 	PedPartition*		part;
 	int			i;
 
-	PED_ASSERT (disk != NULL, return 0);
-	PED_ASSERT (disk->dev != NULL, return 0);
+	PED_ASSERT (disk != NULL);
+	PED_ASSERT (disk->dev != NULL);
 
 	void *s0;
 	if (!ptt_read_sector (disk->dev, 0, &s0))
@@ -1247,6 +1320,7 @@ msdos_partition_new (const PedDisk* disk, PedPartitionType part_type,
 		dos_data->system = PARTITION_LINUX;
 		dos_data->hidden = 0;
 		dos_data->boot = 0;
+		dos_data->diag = 0;
 		dos_data->raid = 0;
 		dos_data->lvm = 0;
 		dos_data->lba = 0;
@@ -1280,6 +1354,7 @@ msdos_partition_duplicate (const PedPartition* part)
 	new_dos_data = (DosPartitionData*) new_part->disk_specific;
 	new_dos_data->system = old_dos_data->system;
 	new_dos_data->boot = old_dos_data->boot;
+	new_dos_data->diag = old_dos_data->diag;
 	new_dos_data->hidden = old_dos_data->hidden;
 	new_dos_data->raid = old_dos_data->raid;
 	new_dos_data->lvm = old_dos_data->lvm;
@@ -1303,7 +1378,7 @@ msdos_partition_duplicate (const PedPartition* part)
 static void
 msdos_partition_destroy (PedPartition* part)
 {
-	PED_ASSERT (part != NULL, return);
+	PED_ASSERT (part != NULL);
 
 	if (ped_partition_is_active (part)) {
 		DosPartitionData* dos_data;
@@ -1329,6 +1404,7 @@ msdos_partition_set_system (PedPartition* part,
 		dos_data->hidden = 0;
 
 	if (part->type & PED_PARTITION_EXTENDED) {
+		dos_data->diag = 0;
 		dos_data->raid = 0;
 		dos_data->lvm = 0;
 		dos_data->palo = 0;
@@ -1340,6 +1416,15 @@ msdos_partition_set_system (PedPartition* part,
 		return 1;
 	}
 
+	if (dos_data->diag) {
+		/* Don't change the system if it already is a diag type,
+		   otherwise use Compaq as almost all vendors use that. */
+		if (dos_data->system != PARTITION_COMPAQ_DIAG &&
+		    dos_data->system != PARTITION_MSFT_RECOVERY &&
+		    dos_data->system != PARTITION_DELL_DIAG)
+			dos_data->system = PARTITION_COMPAQ_DIAG;
+		return 1;
+	}
 	if (dos_data->lvm) {
 		dos_data->system = PARTITION_LINUX_LVM;
 		return 1;
@@ -1371,7 +1456,10 @@ msdos_partition_set_system (PedPartition* part,
 		   || !strcmp (fs_type->name, "hpfs")) {
 		dos_data->system = PARTITION_NTFS;
 		dos_data->system |= dos_data->hidden ? PART_FLAG_HIDDEN : 0;
-	} else if (!strcmp (fs_type->name, "sun-ufs"))
+	} else if (!strcmp (fs_type->name, "hfs")
+		   || !strcmp (fs_type->name, "hfs+"))
+		dos_data->system = PARTITION_HFS;
+	else if (!strcmp (fs_type->name, "sun-ufs"))
 		dos_data->system = PARTITION_SUN_UFS;
 	else if (is_linux_swap (fs_type->name))
 		dos_data->system = PARTITION_LINUX_SWAP;
@@ -1381,6 +1469,17 @@ msdos_partition_set_system (PedPartition* part,
 	return 1;
 }
 
+static void
+clear_flags (DosPartitionData *dos_data)
+{
+  dos_data->diag = 0;
+  dos_data->hidden = 0;
+  dos_data->lvm = 0;
+  dos_data->palo = 0;
+  dos_data->prep = 0;
+  dos_data->raid = 0;
+}
+
 static int
 msdos_partition_set_flag (PedPartition* part,
                           PedPartitionFlag flag, int state)
@@ -1389,9 +1488,9 @@ msdos_partition_set_flag (PedPartition* part,
 	PedPartition*			walk;
 	DosPartitionData*		dos_data;
 
-	PED_ASSERT (part != NULL, return 0);
-	PED_ASSERT (part->disk_specific != NULL, return 0);
-	PED_ASSERT (part->disk != NULL, return 0);
+	PED_ASSERT (part != NULL);
+	PED_ASSERT (part->disk_specific != NULL);
+	PED_ASSERT (part->disk != NULL);
 
 	dos_data = part->disk_specific;
 	disk = part->disk;
@@ -1422,23 +1521,21 @@ msdos_partition_set_flag (PedPartition* part,
 		}
 		return 1;
 
+	case PED_PARTITION_DIAG:
+		if (state)
+			clear_flags (dos_data);
+		dos_data->diag = state;
+		return ped_partition_set_system (part, part->fs_type);
+
 	case PED_PARTITION_RAID:
-		if (state) {
-			dos_data->hidden = 0;
-			dos_data->lvm = 0;
-			dos_data->palo = 0;
-			dos_data->prep = 0;
-		}
+		if (state)
+			clear_flags (dos_data);
 		dos_data->raid = state;
 		return ped_partition_set_system (part, part->fs_type);
 
 	case PED_PARTITION_LVM:
-		if (state) {
-			dos_data->hidden = 0;
-			dos_data->raid = 0;
-			dos_data->palo = 0;
-			dos_data->prep = 0;
-		}
+		if (state)
+			clear_flags (dos_data);
 		dos_data->lvm = state;
 		return ped_partition_set_system (part, part->fs_type);
 
@@ -1447,20 +1544,14 @@ msdos_partition_set_flag (PedPartition* part,
 		return ped_partition_set_system (part, part->fs_type);
 
 	case PED_PARTITION_PALO:
-		if (state) {
-			dos_data->hidden = 0;
-			dos_data->raid = 0;
-			dos_data->lvm = 0;
-		}
+		if (state)
+			clear_flags (dos_data);
 		dos_data->palo = state;
 		return ped_partition_set_system (part, part->fs_type);
 
 	case PED_PARTITION_PREP:
-		if (state) {
-			dos_data->hidden = 0;
-			dos_data->raid = 0;
-			dos_data->lvm = 0;
-		}
+		if (state)
+			clear_flags (dos_data);
 		dos_data->prep = state;
 		return ped_partition_set_system (part, part->fs_type);
 
@@ -1474,17 +1565,23 @@ msdos_partition_get_flag (const PedPartition* part, PedPartitionFlag flag)
 {
 	DosPartitionData*	dos_data;
 
-	PED_ASSERT (part != NULL, return 0);
-	PED_ASSERT (part->disk_specific != NULL, return 0);
+	PED_ASSERT (part != NULL);
+	PED_ASSERT (part->disk_specific != NULL);
 
 	dos_data = part->disk_specific;
 	switch (flag) {
 	case PED_PARTITION_HIDDEN:
-		return dos_data->hidden;
+		if (part->type == PED_PARTITION_EXTENDED)
+			return 0;
+		else
+			return dos_data->hidden;
 
 	case PED_PARTITION_BOOT:
 		return dos_data->boot;
 
+	case PED_PARTITION_DIAG:
+		return dos_data->diag;
+
 	case PED_PARTITION_RAID:
 		return dos_data->raid;
 
@@ -1511,12 +1608,18 @@ msdos_partition_is_flag_available (const PedPartition* part,
 {
 	switch (flag) {
 	case PED_PARTITION_HIDDEN:
+		if (part->type == PED_PARTITION_EXTENDED)
+			return 0;
+		else
+			return 1;
+
 	case PED_PARTITION_BOOT:
 	case PED_PARTITION_RAID:
 	case PED_PARTITION_LVM:
 	case PED_PARTITION_LBA:
 	case PED_PARTITION_PALO:
 	case PED_PARTITION_PREP:
+	case PED_PARTITION_DIAG:
 		return 1;
 
 	default:
@@ -1619,8 +1722,13 @@ _primary_constraint (const PedDisk* disk, const PedCHSGeometry* bios_geom,
 			       		dev->length - min_geom->end))
 			return NULL;
 	} else {
-		if (!ped_geometry_init (&start_geom, dev, cylinder_size,
-			       		dev->length - cylinder_size))
+		/* Use cylinder_size as the starting sector number
+		   when the device is large enough to accommodate that.
+		   Otherwise, use sector 1.  */
+		PedSector start = (cylinder_size < dev->length
+				   ? cylinder_size : 1);
+		if (!ped_geometry_init (&start_geom, dev, start,
+					dev->length - start))
 			return NULL;
 		if (!ped_geometry_init (&end_geom, dev, 0, dev->length))
 			return NULL;
@@ -1701,7 +1809,7 @@ _logical_constraint (const PedDisk* disk, const PedCHSGeometry* bios_geom,
 	PedAlignment	end_align;
 	PedGeometry	max_geom;
 
-	PED_ASSERT (ext_part != NULL, return NULL);
+	PED_ASSERT (ext_part != NULL);
 
 	if (!ped_alignment_init (&start_align, start_offset, cylinder_size))
 		return NULL;
@@ -1752,7 +1860,11 @@ _get_min_extended_part_geom (const PedPartition* ext_part,
 	min_geom = ped_geometry_duplicate (&walk->geom);
 	if (!min_geom)
 		return NULL;
-	ped_geometry_set_start (min_geom, walk->geom.start - 1 * head_size);
+	/* We must always allow at least two sectors at the start, to leave
+	 * room for LILO.  See linux/fs/partitions/msdos.c.
+	 */
+	ped_geometry_set_start (min_geom,
+				walk->geom.start - PED_MAX (1 * head_size, 2));
 
 	for (walk = ext_part->part_list; walk; walk = walk->next) {
 		if (!ped_partition_is_active (walk) || walk->num == 5)
@@ -1824,7 +1936,7 @@ _logical_min_start_head (const PedPartition* part,
 
 /* Shamelessly copied and adapted from _partition_get_overlap_constraint
  * (in disk.c)
- * This should get ride of the infamous Assertion (metadata_length > 0) failed
+ * This should get rid of the infamous Assertion (metadata_length > 0) failed
  * bug for extended msdos disklabels generated by Parted.
  * 1) There always is a partition table at the start of ext_part, so we leave
  *    a one sector gap there.
@@ -1846,7 +1958,7 @@ _log_meta_overlap_constraint (PedPartition* part, const PedGeometry* geom)
 	PedPartition*	walk;
 	int		not_5 = (part->num != 5);
 
-	PED_ASSERT (ext_part != NULL, return NULL);
+	PED_ASSERT (ext_part != NULL);
 
 	walk = ext_part->part_list;
 
@@ -1888,7 +2000,7 @@ _align_logical (PedPartition* part, const PedCHSGeometry* bios_geom,
 	PedGeometry*	solution = NULL;
 	PedConstraint   *intersect, *log_meta_overlap;
 
-	PED_ASSERT (ext_part != NULL, return 0);
+	PED_ASSERT (ext_part != NULL);
 
 	log_meta_overlap = _log_meta_overlap_constraint(part, &part->geom);
 	intersect = ped_constraint_intersect (constraint, log_meta_overlap);
@@ -2026,8 +2138,8 @@ msdos_partition_align (PedPartition* part, const PedConstraint* constraint)
 	PedCHSGeometry	bios_geom;
 	DosPartitionData* dos_data;
 
-	PED_ASSERT (part != NULL, return 0);
-	PED_ASSERT (part->disk_specific != NULL, return 0);
+	PED_ASSERT (part != NULL);
+	PED_ASSERT (part->disk_specific != NULL);
 
 	dos_data = part->disk_specific;
 
@@ -2072,7 +2184,7 @@ add_metadata_part (PedDisk* disk, PedPartitionType type, PedSector start,
 {
 	PedPartition*		new_part;
 
-	PED_ASSERT (disk != NULL, return 0);
+	PED_ASSERT (disk != NULL);
 
 	new_part = ped_partition_new (disk, type | PED_PARTITION_METADATA, NULL,
 				      start, end);
@@ -2129,7 +2241,7 @@ add_logical_part_metadata (PedDisk* disk, const PedPartition* log_part)
 	if (log_part->num == 5 && metadata_length < bios_geom.sectors)
 		return 1;
 
-	PED_ASSERT (metadata_length > 0, return 0);
+	PED_ASSERT (metadata_length > 0);
 
 	return add_metadata_part (disk, PED_PARTITION_LOGICAL,
 				  metadata_start, metadata_end);
@@ -2243,8 +2355,8 @@ msdos_alloc_metadata (PedDisk* disk)
 {
 	PedPartition*		ext_part;
 
-	PED_ASSERT (disk != NULL, return 0);
-	PED_ASSERT (disk->dev != NULL, return 0);
+	PED_ASSERT (disk != NULL);
+	PED_ASSERT (disk->dev != NULL);
 
 	if (!add_startend_metadata (disk))
 		return 0;
@@ -2302,8 +2414,8 @@ next_logical (const PedDisk* disk)
 static int
 msdos_partition_enumerate (PedPartition* part)
 {
-	PED_ASSERT (part != NULL, return 0);
-	PED_ASSERT (part->disk != NULL, return 0);
+	PED_ASSERT (part != NULL);
+	PED_ASSERT (part->disk != NULL);
 
 	/* don't re-number a primary partition */
 	if (part->num != -1 && part->num <= DOS_N_PRI_PARTITIONS)
@@ -2336,7 +2448,7 @@ msdos_get_max_supported_partition_count(const PedDisk* disk, int *max_n)
 PT_define_limit_functions (msdos)
 
 static PedDiskOps msdos_disk_ops = {
-	clobber:		NULL_IF_DISCOVER_ONLY (msdos_clobber),
+	clobber:		NULL,
 	write:			NULL_IF_DISCOVER_ONLY (msdos_write),
 
 	disk_set_flag:          msdos_disk_set_flag,
@@ -2359,8 +2471,8 @@ static PedDiskType msdos_disk_type = {
 void
 ped_disk_msdos_init ()
 {
-	PED_ASSERT (sizeof (DosRawPartition) == 16, return);
-	PED_ASSERT (sizeof (DosRawTable) == 512, return);
+	PED_ASSERT (sizeof (DosRawPartition) == 16);
+	PED_ASSERT (sizeof (DosRawTable) == 512);
 
 	ped_disk_type_register (&msdos_disk_type);
 }
