@@ -2,11 +2,11 @@
 
     libparted - a library for manipulating disk partitions
     disk_amiga.c - libparted module to manipulate amiga RDB partition tables.
-    Copyright (C) 2000, 2001, 2004, 2007 Free Software Foundation, Inc.
+    Copyright (C) 2000-2001, 2004, 2007-2011 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
     Contributor:  Sven Luther <luther@debian.org>
 */
@@ -27,6 +26,12 @@
 #include <parted/debug.h>
 #include <parted/endian.h>
 
+#include "pt-tools.h"
+
+#ifndef MAX
+# define MAX(a,b) ((a) < (b) ? (b) : (a))
+#endif
+
 #if ENABLE_NLS
 #  include <libintl.h>
 #  define _(String) dgettext (PACKAGE, String)
@@ -34,6 +39,8 @@
 #  define _(String) (String)
 #endif /* ENABLE_NLS */
 
+#include "misc.h"
+
 /* String manipulation */
 static void _amiga_set_bstr (const char *cstr, char *bstr, int maxsize) {
 	int size = strlen (cstr);
@@ -46,7 +53,7 @@ static void _amiga_set_bstr (const char *cstr, char *bstr, int maxsize) {
 static const char * _amiga_get_bstr (char * bstr) {
 	char * cstr = bstr + 1;
 	int size = bstr[0];
-	
+
 	cstr[size] = '\0';
 	return cstr;
 }
@@ -57,7 +64,7 @@ static const char * _amiga_get_bstr (char * bstr) {
 #define IDNAME_FILESYSHEADER	(uint32_t)0x46534844	/* 'FSHD' */
 #define IDNAME_LOADSEG		(uint32_t)0x4C534547	/* 'LSEG' */
 #define IDNAME_BOOT		(uint32_t)0x424f4f54	/* 'BOOT' */
-#define IDNAME_FREE		(uint32_t)0xffffffff	
+#define IDNAME_FREE		(uint32_t)0xffffffff
 
 static const char *
 _amiga_block_id (uint32_t id) {
@@ -103,7 +110,7 @@ _amiga_free_ids (struct AmigaIds *ids) {
 
 	for (current = ids; current != NULL; current = next) {
 		next = current->next;
-		ped_free (current);
+		free (current);
 	}
 }
 static int
@@ -122,7 +129,7 @@ struct AmigaBlock {
 	uint32_t	amiga_SummedLongss;	/* Size of the structure for checksums */
 	int32_t		amiga_ChkSum;		/* Checksum of the structure */
 };
-#define AMIGA(pos) ((struct AmigaBlock *)(pos)) 
+#define AMIGA(pos) ((struct AmigaBlock *)(pos))
 
 static int
 _amiga_checksum (struct AmigaBlock *blk) {
@@ -145,7 +152,7 @@ _amiga_calculate_checksum (struct AmigaBlock *blk) {
 	blk->amiga_ChkSum = PED_CPU_TO_BE32(
 		PED_BE32_TO_CPU(blk->amiga_ChkSum) -
 		_amiga_checksum((struct AmigaBlock *) blk));
-	return;	
+	return;
 }
 
 static struct AmigaBlock *
@@ -170,7 +177,7 @@ _amiga_read_block (const PedDevice *dev, struct AmigaBlock *blk,
 					return NULL;
 			case PED_EXCEPTION_IGNORE :
 			case PED_EXCEPTION_UNHANDLED :
-			default : 
+			default :
 				return blk;
 		}
 	}
@@ -222,7 +229,7 @@ struct RigidDiskBlock {
     uint32_t	rdb_Reserved5[10];
 };
 
-#define RDSK(pos) ((struct RigidDiskBlock *)(pos)) 
+#define RDSK(pos) ((struct RigidDiskBlock *)(pos))
 
 #define AMIGA_RDB_NOT_FOUND ((uint32_t)0xffffffff)
 #define	RDB_LOCATION_LIMIT	16
@@ -325,16 +332,16 @@ amiga_probe (const PedDevice *dev)
 {
 	struct RigidDiskBlock *rdb;
 	uint32_t found;
-	PED_ASSERT(dev != NULL, return 0);
+	PED_ASSERT(dev != NULL);
 
 	if ((rdb=RDSK(ped_malloc(dev->sector_size)))==NULL)
 		return 0;
 	found = _amiga_find_rdb (dev, rdb);
-	ped_free (rdb);
+	free (rdb);
 
 	return (found == AMIGA_RDB_NOT_FOUND ? 0 : 1);
 }
-		
+
 static PedDisk*
 amiga_alloc (const PedDevice* dev)
 {
@@ -343,24 +350,28 @@ amiga_alloc (const PedDevice* dev)
 	PedSector cyl_size;
 	int highest_cylinder, highest_block;
 
-	PED_ASSERT(dev != NULL, return NULL);
+	PED_ASSERT(dev != NULL);
 	cyl_size = dev->hw_geom.sectors * dev->hw_geom.heads;
 
 	if (!(disk = _ped_disk_alloc (dev, &amiga_disk_type)))
 		return NULL;
 
-	if (!(disk->disk_specific = ped_malloc (PED_SECTOR_SIZE_DEFAULT))) {
-		ped_free (disk);
+	if (!(disk->disk_specific = ped_malloc (disk->dev->sector_size))) {
+		free (disk);
 		return NULL;
 	}
 	rdb = disk->disk_specific;
 
-	memset(rdb, 0, sizeof(struct RigidDiskBlock));
+        /* Upon failed assertion this does leak.  That's fine, because
+           if the assertion fails, you have bigger problems than this leak. */
+        PED_ASSERT(sizeof(*rdb) <= disk->dev->sector_size);
+
+	memset(rdb, 0, disk->dev->sector_size);
 
 	rdb->rdb_ID = PED_CPU_TO_BE32 (IDNAME_RIGIDDISK);
 	rdb->rdb_SummedLongs = PED_CPU_TO_BE32 (64);
 	rdb->rdb_HostID = PED_CPU_TO_BE32 (0);
-	rdb->rdb_BlockBytes = PED_CPU_TO_BE32 (PED_SECTOR_SIZE_DEFAULT);
+	rdb->rdb_BlockBytes = PED_CPU_TO_BE32 (disk->dev->sector_size);
 	rdb->rdb_Flags = PED_CPU_TO_BE32 (0);
 
 	/* Block lists */
@@ -369,7 +380,7 @@ amiga_alloc (const PedDevice* dev)
 	rdb->rdb_FileSysHeaderList = PED_CPU_TO_BE32 (LINK_END);
 	rdb->rdb_DriveInit = PED_CPU_TO_BE32 (LINK_END);
 	rdb->rdb_BootBlockList = PED_CPU_TO_BE32 (LINK_END);
-	
+
 	/* Physical drive characteristics */
 	rdb->rdb_Cylinders = PED_CPU_TO_BE32 (dev->hw_geom.cylinders);
 	rdb->rdb_Sectors = PED_CPU_TO_BE32 (dev->hw_geom.sectors);
@@ -403,7 +414,7 @@ amiga_alloc (const PedDevice* dev)
 
 	/* And calculate the checksum */
 	_amiga_calculate_checksum ((struct AmigaBlock *) rdb);
-	
+
 	return disk;
 }
 
@@ -413,12 +424,12 @@ amiga_duplicate (const PedDisk* disk)
 	PedDisk*	new_disk;
 	struct RigidDiskBlock *	new_rdb;
 	struct RigidDiskBlock * old_rdb;
-	PED_ASSERT(disk != NULL, return NULL);
-	PED_ASSERT(disk->dev != NULL, return NULL);
-	PED_ASSERT(disk->disk_specific != NULL, return NULL);
+	PED_ASSERT(disk != NULL);
+	PED_ASSERT(disk->dev != NULL);
+	PED_ASSERT(disk->disk_specific != NULL);
 
 	old_rdb = (struct RigidDiskBlock *) disk->disk_specific;
-       
+
 	if (!(new_disk = ped_disk_new_fresh (disk->dev, &amiga_disk_type)))
 		return NULL;
 
@@ -430,36 +441,13 @@ amiga_duplicate (const PedDisk* disk)
 static void
 amiga_free (PedDisk* disk)
 {
-	PED_ASSERT(disk != NULL, return);
-	PED_ASSERT(disk->disk_specific != NULL, return);
+	PED_ASSERT(disk != NULL);
+	PED_ASSERT(disk->disk_specific != NULL);
 
-	ped_free (disk->disk_specific);
+	free (disk->disk_specific);
 	_ped_disk_free (disk);
 }
 
-#ifndef DISCOVER_ONLY
-static int
-amiga_clobber (PedDevice* dev)
-{
-	struct RigidDiskBlock *rdb;
-	uint32_t i;
-	int result = 0;
-	PED_ASSERT(dev != NULL, return 0);
-
-	if ((rdb=RDSK(ped_malloc(PED_SECTOR_SIZE_DEFAULT)))==NULL)
-		return 0;
-
-	while ((i = _amiga_find_rdb (dev, rdb)) != AMIGA_RDB_NOT_FOUND) {
-		rdb->rdb_ID = PED_CPU_TO_BE32 (0);
-		result = ped_device_write (dev, (void*) rdb, i, 1);
-	}
-
-	ped_free (rdb);
-
-	return result;
-}
-#endif /* !DISCOVER_ONLY */
-
 static int
 _amiga_loop_check (uint32_t block, uint32_t * blocklist, uint32_t max)
 {
@@ -485,11 +473,10 @@ amiga_read (PedDisk* disk)
 	PedSector cylblocks;
 	int i;
 
-	PED_ASSERT(disk != NULL, return 0);
-	PED_ASSERT(disk->dev != NULL, return 0);
-	PED_ASSERT(disk->dev->sector_size % PED_SECTOR_SIZE_DEFAULT == 0,
-                   return 0);
-	PED_ASSERT(disk->disk_specific != NULL, return 0);
+	PED_ASSERT(disk != NULL);
+	PED_ASSERT(disk->dev != NULL);
+	PED_ASSERT(disk->dev->sector_size % PED_SECTOR_SIZE_DEFAULT == 0);
+	PED_ASSERT(disk->disk_specific != NULL);
 	rdb = RDSK(disk->disk_specific);
 
 	if (_amiga_find_rdb (disk->dev, rdb) == AMIGA_RDB_NOT_FOUND) {
@@ -532,7 +519,7 @@ amiga_read (PedDisk* disk)
 		/* Let's allocate and read a partition block to get its geometry*/
 		if (!_amiga_read_block (disk->dev, AMIGA(partition),
 		                        (PedSector)partblock, NULL)) {
-			ped_free(partition);
+			free(partition);
 			return 0;
 		}
 
@@ -542,8 +529,9 @@ amiga_read (PedDisk* disk)
 			+ 1) * cylblocks - 1;
 
 		/* We can now construct a new partition */
-		if (!(part = ped_partition_new (disk, 0, NULL, start, end))) {
-			ped_free(partition);
+		if (!(part = ped_partition_new (disk, PED_PARTITION_NORMAL,
+                                                NULL, start, end))) {
+			free(partition);
 			return 0;
 		}
 		/* And copy over the partition block */
@@ -553,15 +541,16 @@ amiga_read (PedDisk* disk)
 		part->type = 0;
 		/* Let's probe what file system is present on the disk */
 		part->fs_type = ped_file_system_probe (&part->geom);
-		
+
 		constraint_exact = ped_constraint_exact (&part->geom);
 		if (!ped_disk_add_partition (disk, part, constraint_exact)) {
 			ped_partition_destroy(part);
-			ped_free(partition);
+			free(partition);
 			return 0;
 		}
 		ped_constraint_destroy (constraint_exact);
 	}
+	free(partition);
 	return 1;
 }
 
@@ -571,8 +560,8 @@ _amiga_find_free_blocks(const PedDisk *disk, uint32_t *table,
 {
 	PedSector next;
 
-	PED_ASSERT(disk != NULL, return 0);
-	PED_ASSERT(disk->dev != NULL, return 0);
+	PED_ASSERT(disk != NULL);
+	PED_ASSERT(disk->dev != NULL);
 
 	for (next = first; next != LINK_END; next = PED_BE32_TO_CPU(block->lk_Next)) {
 		if (table[next] != IDNAME_FREE) {
@@ -586,7 +575,7 @@ _amiga_find_free_blocks(const PedDisk *disk, uint32_t *table,
 					/* TODO : Need to add fixing code */
 				case PED_EXCEPTION_IGNORE :
 				case PED_EXCEPTION_UNHANDLED :
-				default : 
+				default :
 					return 1;
 			}
 		}
@@ -603,7 +592,7 @@ _amiga_find_free_blocks(const PedDisk *disk, uint32_t *table,
 				/* TODO : to more subtile things here */
 				case PED_EXCEPTION_CANCEL :
 				case PED_EXCEPTION_UNHANDLED :
-				default : 
+				default :
 					return 0;
 			}
 		}
@@ -640,43 +629,53 @@ amiga_write (const PedDisk* disk)
 	struct LinkedBlock *block;
 	struct PartitionBlock *partition;
 	PedPartition *part, *next_part;
-	PedSector cylblocks, first_hb, last_hb, last_used_hb;
+	PedSector cylblocks, first_hb, last_hb;
 	uint32_t * table;
 	uint32_t i;
 	uint32_t rdb_num, part_num, block_num, next_num;
 
-	PED_ASSERT (disk != NULL, return 0);
-	PED_ASSERT (disk->dev != NULL, return 0);
-	PED_ASSERT (disk->disk_specific != NULL, return 0);
+	PED_ASSERT (disk != NULL);
+	PED_ASSERT (disk->dev != NULL);
+	PED_ASSERT (disk->disk_specific != NULL);
 
-	if (!(rdb = ped_malloc (PED_SECTOR_SIZE_DEFAULT)))
+	if (!(rdb = ped_malloc (disk->dev->sector_size)))
 		return 0;
 
 	/* Let's read the rdb */
 	if ((rdb_num = _amiga_find_rdb (disk->dev, rdb)) == AMIGA_RDB_NOT_FOUND) {
 		rdb_num = 2;
+		size_t pb_size = sizeof (struct PartitionBlock);
+                /* Initialize only the part that won't be copied over
+                   with a partition block in amiga_read.  */
+		memset ((char *)(RDSK(disk->disk_specific)) + pb_size,
+			0, PED_SECTOR_SIZE_DEFAULT - pb_size);
 	} else {
-		memcpy (RDSK(disk->disk_specific), rdb, PED_SECTOR_SIZE_DEFAULT);
+		memcpy (RDSK(disk->disk_specific), rdb, disk->dev->sector_size);
 	}
-	ped_free (rdb);
+	free (rdb);
 	rdb = RDSK(disk->disk_specific);
 
 	cylblocks = (PedSector) PED_BE32_TO_CPU (rdb->rdb_Heads) *
 		(PedSector) PED_BE32_TO_CPU (rdb->rdb_Sectors);
 	first_hb = (PedSector) PED_BE32_TO_CPU (rdb->rdb_RDBBlocksLo);
 	last_hb = (PedSector) PED_BE32_TO_CPU (rdb->rdb_RDBBlocksHi);
-	last_used_hb = (PedSector) PED_BE32_TO_CPU (rdb->rdb_HighRDSKBlock);
 
-	/* let's allocate a free block table and initialize it */
-	if (!(table = ped_malloc ((last_hb - first_hb + 1) * sizeof(uint32_t))))
+	/* Allocate a free block table and initialize it.
+	   There must be room for at least RDB_NUM + 2 entries, since
+	   the first RDB_NUM+1 entries get IDNAME_RIGIDDISK, and the
+	   following one must have LINK_END to serve as sentinel.  */
+	size_t tab_size = 2 + MAX (last_hb - first_hb, rdb_num);
+	if (!(table = ped_malloc (tab_size * sizeof *table)))
 		return 0;
 
-	memset(table, 0xff, (last_hb - first_hb + 1) * sizeof(uint32_t));
-	for (i = 0; i<=rdb_num; i++) table[i] = IDNAME_RIGIDDISK;
-	
+	for (i = 0; i <= rdb_num; i++)
+		table[i] = IDNAME_RIGIDDISK;
+	for (     ; i < tab_size; i++)
+		table[i] = LINK_END;
+
 	/* Let's allocate a partition block */
-	if (!(block = ped_malloc (PED_SECTOR_SIZE_DEFAULT))) {
-		ped_free (table);
+	if (!(block = ped_malloc (disk->dev->sector_size))) {
+		free (table);
 		return 0;
 	}
 
@@ -714,14 +713,14 @@ amiga_write (const PedDisk* disk)
 		goto error_free_table;
 	}
 
-	block_num = next_num = part_num = _amiga_next_free_block(table, rdb_num+1,
-	                                                         IDNAME_PARTITION);
+	block_num = part_num = _amiga_next_free_block(table, rdb_num+1,
+                                                      IDNAME_PARTITION);
 	part = _amiga_next_real_partition(disk, NULL);
 	rdb->rdb_PartitionList = PED_CPU_TO_BE32(part ? part_num : LINK_END);
 	for (; part != NULL; part = next_part, block_num = next_num) {
-		PED_ASSERT(part->disk_specific != NULL, return 0);
-		PED_ASSERT(part->geom.start % cylblocks == 0, return 0);
-		PED_ASSERT((part->geom.end + 1) % cylblocks == 0, return 0);
+		PED_ASSERT(part->disk_specific != NULL);
+		PED_ASSERT(part->geom.start % cylblocks == 0);
+		PED_ASSERT((part->geom.end + 1) % cylblocks == 0);
 
 		next_part = _amiga_next_real_partition(disk, part);
 		next_num = _amiga_next_free_block(table, block_num+1, IDNAME_PARTITION);
@@ -756,13 +755,13 @@ amiga_write (const PedDisk* disk)
 	if (!ped_device_write (disk->dev, (void*) disk->disk_specific, rdb_num, 1))
 		goto error_free_table;
 
-	ped_free (table);
-	ped_free (block);
+	free (table);
+	free (block);
 	return ped_device_sync (disk->dev);
 
 error_free_table:
-	ped_free (table);
-	ped_free (block);
+	free (table);
+	free (block);
 	return 0;
 }
 #endif /* !DISCOVER_ONLY */
@@ -778,9 +777,9 @@ amiga_partition_new (const PedDisk* disk, PedPartitionType part_type,
 	struct PartitionBlock *partition;
 	struct RigidDiskBlock *rdb;
 
-	PED_ASSERT(disk != NULL, return NULL);
-	PED_ASSERT(disk->dev != NULL, return NULL);
-	PED_ASSERT(disk->disk_specific != NULL, return NULL);
+	PED_ASSERT(disk != NULL);
+	PED_ASSERT(disk->dev != NULL);
+	PED_ASSERT(disk->disk_specific != NULL);
 	dev = disk->dev;
 	cyl = (PedSector) (dev->hw_geom.sectors * dev->hw_geom.heads);
 	rdb = RDSK(disk->disk_specific);
@@ -789,8 +788,8 @@ amiga_partition_new (const PedDisk* disk, PedPartitionType part_type,
 		return NULL;
 
 	if (ped_partition_is_active (part)) {
-		if (!(part->disk_specific = ped_malloc (PED_SECTOR_SIZE_DEFAULT))) {
-			ped_free (part);
+		if (!(part->disk_specific = ped_malloc (disk->dev->sector_size))) {
+			free (part);
 			return NULL;
 		}
 		partition = PART(part->disk_specific);
@@ -839,9 +838,9 @@ amiga_partition_duplicate (const PedPartition* part)
 	struct PartitionBlock *new_amiga_part;
 	struct PartitionBlock *old_amiga_part;
 
-	PED_ASSERT(part != NULL, return NULL);
-	PED_ASSERT(part->disk != NULL, return NULL);
-	PED_ASSERT(part->disk_specific != NULL, return NULL);
+	PED_ASSERT(part != NULL);
+	PED_ASSERT(part->disk != NULL);
+	PED_ASSERT(part->disk_specific != NULL);
 	old_amiga_part = (struct PartitionBlock *) part->disk_specific;
 
 	new_part = ped_partition_new (part->disk, part->type,
@@ -859,11 +858,11 @@ amiga_partition_duplicate (const PedPartition* part)
 static void
 amiga_partition_destroy (PedPartition* part)
 {
-	PED_ASSERT (part != NULL, return);
+	PED_ASSERT (part != NULL);
 
 	if (ped_partition_is_active (part)) {
-		PED_ASSERT (part->disk_specific != NULL, return);
-		ped_free (part->disk_specific);
+		PED_ASSERT (part->disk_specific != NULL);
+		free (part->disk_specific);
 	}
 	_ped_partition_free (part);
 }
@@ -874,8 +873,8 @@ amiga_partition_set_system (PedPartition* part,
 {
 	struct PartitionBlock *partition;
 
-	PED_ASSERT (part != NULL, return 0);
-	PED_ASSERT (part->disk_specific != NULL, return 0);
+	PED_ASSERT (part != NULL);
+	PED_ASSERT (part->disk_specific != NULL);
 	partition = PART(part->disk_specific);
 
 	part->fs_type = fs_type;
@@ -886,7 +885,7 @@ amiga_partition_set_system (PedPartition* part,
 		partition->de_DosType = PED_CPU_TO_BE32(0x4c4e5800); /* 'LNX\0' */
 	else if (!strcmp (fs_type->name, "ext3"))
 		partition->de_DosType = PED_CPU_TO_BE32(0x45585403); /* 'EXT\3' */
-	else if (!strcmp (fs_type->name, "linux-swap"))
+	else if (is_linux_swap (fs_type->name))
 		partition->de_DosType = PED_CPU_TO_BE32(0x53575000); /* 'SWP\0' */
 	else if (!strcmp (fs_type->name, "fat16"))
 		partition->de_DosType = PED_CPU_TO_BE32(0x46415400); /* 'FAT\0' */
@@ -916,8 +915,8 @@ amiga_partition_set_flag (PedPartition* part, PedPartitionFlag flag, int state)
 {
 	struct PartitionBlock *partition;
 
-	PED_ASSERT (part != NULL, return 0);
-	PED_ASSERT (part->disk_specific != NULL, return 0);
+	PED_ASSERT (part != NULL);
+	PED_ASSERT (part->disk_specific != NULL);
 
 	partition = PART(part->disk_specific);
 
@@ -948,8 +947,8 @@ amiga_partition_get_flag (const PedPartition* part, PedPartitionFlag flag)
 {
 	struct PartitionBlock *partition;
 
-	PED_ASSERT (part != NULL, return 0);
-	PED_ASSERT (part->disk_specific != NULL, return 0);
+	PED_ASSERT (part != NULL);
+	PED_ASSERT (part->disk_specific != NULL);
 
 	partition = PART(part->disk_specific);
 
@@ -982,13 +981,13 @@ amiga_partition_is_flag_available (const PedPartition* part,
 	}
 }
 
-static void             
+static void
 amiga_partition_set_name (PedPartition* part, const char* name)
-{               
+{
 	struct PartitionBlock *partition;
 
-	PED_ASSERT (part != NULL, return);
-	PED_ASSERT (part->disk_specific != NULL, return);
+	PED_ASSERT (part != NULL);
+	PED_ASSERT (part->disk_specific != NULL);
 
 	partition = PART(part->disk_specific);
 	_amiga_set_bstr(name, partition->pb_DriveName, 32);
@@ -998,14 +997,23 @@ amiga_partition_get_name (const PedPartition* part)
 {
 	struct PartitionBlock *partition;
 
-	PED_ASSERT (part != NULL, return 0);
-	PED_ASSERT (part->disk_specific != NULL, return 0);
+	PED_ASSERT (part != NULL);
+	PED_ASSERT (part->disk_specific != NULL);
 
 	partition = PART(part->disk_specific);
 
 	return _amiga_get_bstr(partition->pb_DriveName);
 }
 
+static PedAlignment*
+amiga_get_partition_alignment(const PedDisk *disk)
+{
+	PedSector cylinder_size =
+		disk->dev->hw_geom.sectors * disk->dev->hw_geom.heads;
+
+        return ped_alignment_new(0, cylinder_size);
+}
+
 static PedConstraint*
 _amiga_get_constraint (const PedDisk *disk)
 {
@@ -1029,9 +1037,9 @@ _amiga_get_constraint (const PedDisk *disk)
 static int
 amiga_partition_align (PedPartition* part, const PedConstraint* constraint)
 {
-	PED_ASSERT (part != NULL, return 0);
-	PED_ASSERT (part->disk != NULL, return 0);
-	
+	PED_ASSERT (part != NULL);
+	PED_ASSERT (part->disk != NULL);
+
 	if (_ped_partition_attempt_align (part, constraint,
 					  _amiga_get_constraint (part->disk)))
 	       	return 1;
@@ -1048,9 +1056,9 @@ amiga_partition_enumerate (PedPartition* part)
 {
 	int i;
 	PedPartition* p;
-	
-	PED_ASSERT (part != NULL, return 0);
-	PED_ASSERT (part->disk != NULL, return 0);
+
+	PED_ASSERT (part != NULL);
+	PED_ASSERT (part->disk != NULL);
 
 	/* never change the partition numbers */
 	if (part->num != -1)
@@ -1077,8 +1085,8 @@ amiga_alloc_metadata (PedDisk* disk)
 	PedPartition*		new_part;
 	PedConstraint*		constraint_any = NULL;
 
-	PED_ASSERT (disk != NULL, goto error);
-	PED_ASSERT (disk->dev != NULL, goto error);
+	PED_ASSERT (disk != NULL);
+	PED_ASSERT (disk->dev != NULL);
 
 	constraint_any = ped_constraint_any (disk->dev);
 
@@ -1106,40 +1114,26 @@ amiga_get_max_primary_partition_count (const PedDisk* disk)
 	return AMIGA_MAX_PARTITIONS;
 }
 
+static bool
+amiga_get_max_supported_partition_count (const PedDisk* disk, int *max_n)
+{
+	*max_n = AMIGA_MAX_PARTITIONS;
+	return true;
+}
+
+#include "pt-common.h"
+PT_define_limit_functions (amiga)
+
 static PedDiskOps amiga_disk_ops = {
-	probe:			amiga_probe,
-#ifndef DISCOVER_ONLY
-	clobber:		amiga_clobber,
-#else
 	clobber:		NULL,
-#endif
-	alloc:			amiga_alloc,
-	duplicate:		amiga_duplicate,
-	free:			amiga_free,
-	read:			amiga_read,
-#ifndef DISCOVER_ONLY
-	write:			amiga_write,
-#else
-	write:			NULL,
-#endif
+	write:			NULL_IF_DISCOVER_ONLY (amiga_write),
 
-	partition_new:		amiga_partition_new,
-	partition_duplicate:	amiga_partition_duplicate,
-	partition_destroy:	amiga_partition_destroy,
-	partition_set_system:	amiga_partition_set_system,
-	partition_set_flag:	amiga_partition_set_flag,
-	partition_get_flag:	amiga_partition_get_flag,
-	partition_is_flag_available:
-				amiga_partition_is_flag_available,
 	partition_set_name:	amiga_partition_set_name,
 	partition_get_name:	amiga_partition_get_name,
-	partition_align:	amiga_partition_align,
-	partition_enumerate:	amiga_partition_enumerate,
 
+	get_partition_alignment: amiga_get_partition_alignment,
 
-	alloc_metadata:		amiga_alloc_metadata,
-	get_max_primary_partition_count:
-				amiga_get_max_primary_partition_count
+	PT_op_function_initializers (amiga)
 };
 
 static PedDiskType amiga_disk_type = {
@@ -1152,11 +1146,11 @@ static PedDiskType amiga_disk_type = {
 void
 ped_disk_amiga_init ()
 {
-	PED_ASSERT (sizeof (struct AmigaBlock) != 3, return);
-	PED_ASSERT (sizeof (struct RigidDiskBlock) != 64, return);
-	PED_ASSERT (sizeof (struct PartitionBlock) != 64, return);
-	PED_ASSERT (sizeof (struct LinkedBlock) != 5, return);
-	PED_ASSERT (sizeof (struct Linked2Block) != 18, return);
+	PED_ASSERT (sizeof (struct AmigaBlock) != 3);
+	PED_ASSERT (sizeof (struct RigidDiskBlock) != 64);
+	PED_ASSERT (sizeof (struct PartitionBlock) != 64);
+	PED_ASSERT (sizeof (struct LinkedBlock) != 5);
+	PED_ASSERT (sizeof (struct Linked2Block) != 18);
 
 	ped_disk_type_register (&amiga_disk_type);
 }
