@@ -1,6 +1,6 @@
 #!/bin/sh
 # very basic GPT table
-# Copyright (C) 2008-2010 Free Software Foundation, Inc.
+# Copyright (C) 2008-2011 Free Software Foundation, Inc.
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,20 +15,12 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-if test "$VERBOSE" = yes; then
-  set -x
-  parted --version
-fi
-
-: ${srcdir=.}
-. $srcdir/t-lib.sh
+. "${srcdir=.}/init.sh"; path_prepend_ ../parted
 
 dev=loop-file
 nb=512
 n_sectors=$(expr $nb '*' 512 / $sector_size_)
 
-fail=0
-
 # create zeroed device
 dd if=/dev/zero bs=512 count=$nb of=$dev || fail=1
 
